<html>
	<head>
		<title>CoRDial Face</title>
		<style>
		html { overflow-x: hidden; overflow-y: hidden; }
        svg {z-index: -10;}
		</style>
	</head>
	<body>
		<script src="{{ url_for('static', filename='js/socketio.min.js') }}"></script>
        <script src="{{ url_for('static', filename='js/two.min.js') }}"></script>
        <script src="{{ url_for('static', filename='js/tween.min.js') }}"></script>
        <script src="{{ url_for('static', filename='js/svgFace.js') }}"></script>

        <script src="{{ url_for('static', filename='js/dat.gui.min.js') }}"></script>
        <script src="{{ url_for('static', filename='js/facecontrol.js') }}"></script>


		<script>
		  startFace('#D7E4F5', //background color
		  'DB1','', //robot name, ros_master_uri
		  .009, //cm per pixel
		  0, //viseme compression factor
		  '#ffffff', '#800080',150,80,400, 80, .7,"round", //eyes (white_color, iris_color, size, height, separation, iris_size, pupil_scale, pupil_shape)
		  '#D7E4F5', 150,130,.4,  //eyelids (width, height, arch)
		  '#ff99cc', 0,10,0,0, //nose (color, x, y, width, height)
		  '#2c241b', 15,-60,450,20,18,2, 8, 100.2, 1000.2, //mouth (color, x, y, width, height, thickness, opening, dimple_size, ulip_h_scale, llip_h_scale)
		  '#2c241b', 130,120,18,11 //brows (color, width, height, thickness, innersize)
		  )
		</script>
        
	</body>
</html>